--[[
-- ElvUI Location Plus --
a plugin for ElvUI, that adds player location and coords + 2 Datatexts
by Benik @ Emerald Dream EU
- Info, requests, bugs: http://www.tukui.org/addons/index.php?act=view&id=56
----------------------------------------------------------------------------------
- Credits:
	-Elv, Blazeflack, Darth Predator, Talryn, for showing me the best way to do this
	-CohesioN(idea from his TukUI edit)

	-CodeRedLin, iceeagle, daf2(from Broker_Location addon)
	-grdn (from iLocation addon), for digging their great code and making this possible.
	-Tukui and Elvui forum community.
----------------------------------------------------------------------------------
- ToDo:

]]--

local E, L, V, P, G, _ = unpack(ElvUI); --Inport: Engine, Locales, PrivateDB, ProfileDB, GlobalDB, Localize Underscore
local DT = E:GetModule('DataTexts')
local LSM = LibStub("LibSharedMedia-3.0")

local tourist = LibStub("LibTourist-3.0")

local LOCPANEL_HEIGHT = 20 --all panels height
local LOCPANEL_WIDTH = 200 --Location panel width, though its size depends on the location string
local COORDS_WIDTH = 30    --Coord panels width
local DTPANELS_WIDTH = 100 --Datatext panels width

local ElvUI_Location = CreateFrame('Frame', 'ElvUI_Location', E.UIParent)
ElvUI_Location.version = '1.58'

local AddZone; -- Credits grvn
do
	local r, g, b, isPvP, isRaid;
	
	AddZone = function(zone)
		isPvP = nil;
		isRaid = nil;
		
		-- check if it's PvP and filter it, if set in the options
		if( tourist:IsPvPZone(zone) or tourist:IsArena(zone) or tourist:IsBattleground(zone) ) then
			if loc_db.loc_tthidepvp then
				return;
			end
			isPvP = true;
		end
		
		-- check if it's a raid and filter it, if set in the options
		-- if it's a PvP zone, we cancel the check for group size
		if( not isPvP and tourist:GetInstanceGroupSize(zone) >= 10 ) then
			if loc_db.loc_tthideraid then
				return;
			end
			isRaid = true;
		end
		
		r, g, b = tourist:GetLevelColor(zone);
		low, high = tourist:GetLevel(zone);
		GameTooltip:AddDoubleLine("|cffffffff"..zone..(isPvP and "|cffff0000 - ".._G.PVP.."|r" or "")..(isRaid and "|cffff4400 - ".._G.RAID.."|r" or ""), 
		("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255,(low == high and low or ("%d-%d"):format(low, high))));
	end
end

local function LocUpdateTooltip()
	local zoneText = _G.GetRealZoneText() or _G.UNKNOWN;
	
	GameTooltip:ClearLines()
	GameTooltip:AddDoubleLine(L["Zone : "], zoneText, 1, 1, 1, selectioncolor)
	GameTooltip:AddDoubleLine(L["Continent : "], tourist:GetContinent(zoneText), 1, 1, 1, selectioncolor)
	
	-- Status on Tooltip
	if loc_db.loc_ttst then
		local text
		local r, g, b = 1, 1, 0
		pvpType, _, _ = GetZonePVPInfo()
			if (pvpType == "sanctuary") then
				text = L["Sanctuary"]
				r, g, b = 0.41, 0.8, 0.94
			elseif(pvpType == "arena") then
				text = _G.ARENA
				r, g, b = 1, 0.1, 0.1
			elseif(pvpType == "friendly") then
				text = _G.FRIENDLY
				r, g, b = 0.1, 1, 0.1
			elseif(pvpType == "hostile") then
				text = _G.HOSTILE
				r, g, b = 1, 0.1, 0.1
			elseif(pvpType == "contested") then
				text = L["Contested"]
				r, g, b = 1, 0.7, 0.10
			elseif(pvpType == "combat" ) then
				text = _G.COMBAT
				r, g, b = 1, 0.1, 0.1
			else
				text = _G.UNKNOWN
			end
		GameTooltip:AddDoubleLine(_G.STATUS.." :", text, 1, 1, 1, r, g, b)
	end

    -- Zone level range
	if loc_db.loc_ttlvl then
		local low, high = tourist:GetLevel(zoneText)
		if low >= 1 and high >= 1 then
			local r, g, b = tourist:GetLevelColor(zoneText)
			GameTooltip:AddDoubleLine(L["Level Range : "], low == high and low or string.format("%d-%d", low, high), 1, 1, 1, r, g, b)
		end
	end

	-- Recommended zones
	if loc_db.loc_ttreczones then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(L["Recommended Zones :"], selectioncolor)
	
		for zone in tourist:IterateRecommendedZones() do
			AddZone(zone);
		end		
	end
	
	-- Instances in the zone
	if loc_db.loc_ttinst and tourist:DoesZoneHaveInstances(zoneText) then 
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(format(_G.DUNGEONS.." :"), selectioncolor)
			
		for zone in tourist:IterateZoneInstances(zoneText) do
			AddZone(zone);
		end	
	end
	
	-- Recommended Instances
	if loc_db.loc_ttrecinst and tourist:DoesZoneHaveInstances(zoneText) then 
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(L["Recommended Dungeons :"], selectioncolor)
			
		for zone in tourist:IterateRecommendedInstances() do
			AddZone(zone);
		end
	end
	
	-- Hints
	if loc_db.loc_tt then
		if loc_db.loc_tthint then
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine(L["Click : "], L["Toggle WorldMap"],0.7,0.7,1,0.7,0.7,1)
			if (IsAddOnLoaded("Atlas")) then
				GameTooltip:AddDoubleLine(L["RightClick : "], L["Toggle Atlas"],0.7,0.7,1,0.7,0.7,1)
			end
			GameTooltip:AddDoubleLine(L["ShiftClick : "], L["Send position to chat"],0.7,0.7,1,0.7,0.7,1)
			GameTooltip:AddDoubleLine(L["CtrlClick : "], L["Toggle Datatexts"],0.7,0.7,1,0.7,0.7,1)
		end
		GameTooltip:Show()
	else
		GameTooltip:Hide()
	end
	
end

-- mouse over the location panel
local function LocPanel_OnEnter(self,...)
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, -8)
	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint("BOTTOM", self, "BOTTOM", 0, 0)
	LocUpdateTooltip()
end

-- mouse leaving the location panel
local function LocPanel_OnLeave(self,...)
	GameTooltip:Hide()
end

-- Hide in combat, after fade function ends
local function LocPanelOnFade()
	LocationPlusPanel:Hide()
end

-- clicking the location panel
local function LocPanel_OnClick(self, btn)
	zoneText = _G.GetRealZoneText() or _G.UNKNOWN;
	if btn == "LeftButton" then	
		if IsShiftKeyDown() then
			local edit_box = _G.ChatEdit_ChooseBoxForSend()
			local x, y = GetPlayerMapPosition("player")
			local dig
			if loc_db.loc_dig then
				dig = 2
			else
				dig = 0
			end
			x = tonumber(E:Round(100 * x, dig))
			y = tonumber(E:Round(100 * y, dig))
			local message
			local coords = x..", "..y
				if zoneText ~= GetSubZoneText() then
					message = string.format("%s: %s (%s)", zoneText, GetSubZoneText(), coords)
				else
					message = string.format("%s (%s)", zoneText, coords)
				end
			_G.ChatEdit_ActivateChat(edit_box)
			edit_box:Insert(message) 
		else
			if IsControlKeyDown() then
				if loc_db.loc_datatxt then
					ToggleFrame(LeftCoordDtPanel)
					ToggleFrame(RightCoordDtPanel)
				else
					StaticPopup_Show("LOC_DT_DISABLED")
				end
			else 
				ToggleFrame(WorldMapFrame)
			end
		end
	end
	if (IsAddOnLoaded("Atlas")) and btn == "RightButton" then
		Atlas_Toggle()
	end
end

-- Location panel
function CreateMainLocPanel()
	local loc_panel = CreateFrame('Button', 'LocationPlusPanel', ElvUI_Location)
	loc_panel:Width(LOCPANEL_WIDTH)
	loc_panel:SetHeight(loc_db.dtheight)
	loc_panel:Point('TOP', E.UIParent, loc_db.lpx, loc_db.lpy)
	loc_panel:SetFrameStrata('LOW')
	loc_panel:RegisterForClicks('AnyUp')
	loc_panel:SetScript('OnEnter', LocPanel_OnEnter) 
	loc_panel:SetScript('OnLeave', LocPanel_OnLeave)
	loc_panel:SetScript('OnClick', LocPanel_OnClick)
	loc_panel:SetClampedToScreen(true)
	loc_panel:Show()

	-- Location Text
	loc_panel.Text = LocationPlusPanel:CreateFontString(nil, "LOW")
	loc_panel.Text:Point("CENTER", 0, 0)
	loc_panel.Text:SetJustifyH('CENTER')
	loc_panel.Text:SetJustifyV('MIDDLE')
	
	loc_panel:SetScript("OnUpdate", function(self,event,...)
		local subZoneText = GetMinimapZoneText() or ""
		local zoneText = _G.GetRealZoneText() or _G.UNKNOWN;
		local displayLine
		
		-- zone and subzone
		if loc_db.loc_both then
			if (subZoneText ~= "") and (subZoneText ~= zoneText) then
				displayLine = zoneText .. ": " .. subZoneText
			else
				displayLine = subZoneText
			end
		else
			displayLine = subZoneText
		end
		
		local r, g, b = tourist:GetFactionColor(zoneText)
		--local r, g, b = LocMopColoring()
		
		if displayLine ~= "" then
			pvpType = GetZonePVPInfo() -- LibTourist sanctuary coloring fix
			if (pvpType == "sanctuary") then
				self.Text:SetText(string.format("|cff%02x%02x%02x%s|r", 104.55, 204, 239.7, displayLine))
			else
				self.Text:SetText(string.format("|cff%02x%02x%02x%s|r", r*255, g*255, b*255, displayLine))
			end
		end
		
		self:SetWidth(LocationPlusPanel.Text:GetWidth() + 24)
	end)
	
	--Hide in combat
	loc_panel:SetScript("OnEvent",function(self, event)
		if loc_db.loc_combat then
			if event == "PLAYER_REGEN_DISABLED" then
				UIFrameFadeOut(self, 0.2, self:GetAlpha(), 0)
				self.fadeInfo.finishedFunc = LocPanelOnFade
			elseif event == "PLAYER_REGEN_ENABLED" then
				UIFrameFadeIn(self, 0.2, self:GetAlpha(), 1)
				self:Show()
			end
		end
	end)

	-- Mover
	--E:CreateMover(LocationPlusPanel, "LocationMover", "Location Panel")
end

-- LocationPanel Positioning
function LocPlusPosition()
	LocationPlusPanel:ClearAllPoints()
	LocationPlusPanel:Point('TOP', E.UIParent, 'BOTTOMLEFT', loc_db.lpx, loc_db.lpy)
end

-- datatext panels width
function LocPlusDTWidth()
	LeftCoordDtPanel:SetWidth(loc_db.dtwidth)
	RightCoordDtPanel:SetWidth(loc_db.dtwidth)
end

-- all panels height
function LocPlusDTHeight()
	if loc_db.loc_ht then
		LocationPlusPanel:SetHeight((loc_db.dtheight)+6)
	else
		LocationPlusPanel:SetHeight(loc_db.dtheight)
	end
	LeftCoordDtPanel:SetHeight(loc_db.dtheight)
	RightCoordDtPanel:SetHeight(loc_db.dtheight)
	XCoordsPanel:SetHeight(loc_db.dtheight)
	YCoordsPanel:SetHeight(loc_db.dtheight)
end

-- Update changes
function E:LocPlusUpdate()
	AllLocPanelsTransparent()
	AllLocPanelsShadow()
	AllLocPanelsShadowReposition()
	LocPlusDTHeight()
	LocPlusPosition()
	LCoordsDig()
end

-- Fonts --     Credits: Phenox(with support for his addon)
function E:LocCoordPanelFont()

	self["media"].lpFont = LSM:Fetch("font", loc_db.lpfont)
	LocationPlusPanel.Text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)			
	XCoordsPanel.Text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)
	YCoordsPanel.Text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)

	if not (IsAddOnLoaded("ElvUI_Phenox")) then
		local panelsToFont = { RightCoordDtPanel, LeftCoordDtPanel }
		for panelName, panel in pairs(panelsToFont) do
			for i=1, panel.numPoints do
				local pointIndex = DT.PointLocation[i]
				panel.dataPanels[pointIndex].text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)			
			end
		end	
	else return end
	
end

-- Enable/Disable shadows without reloadui, YEY!
function AllLocPanelsShadow()
	local panelsToAddShadow = { LocationPlusPanel, XCoordsPanel, YCoordsPanel, LeftCoordDtPanel, RightCoordDtPanel }
	
	for _, frame in pairs( panelsToAddShadow ) do
	frame:CreateShadow('Default')
		if loc_db.loc_shadow then
			frame.shadow:Show()
		else
			frame.shadow:Hide()
		end
	end
end

-- Reposition when shadows enabled
function AllLocPanelsShadowReposition()
	if loc_db.loc_shadow then
		XCoordsPanel:Point('RIGHT', LocationPlusPanel, 'LEFT', -3, 0)
		XCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', -3, 0)
		YCoordsPanel:Point('LEFT', LocationPlusPanel, 'RIGHT', 3, 0)
		YCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', 3, 0)
		LeftCoordDtPanel:Point('RIGHT', XCoordsPanel, 'LEFT', -3, 0)
		LeftCoordDtPanel:Point('CENTER', XCoordsPanel, 'CENTER', -3, 0)
		RightCoordDtPanel:Point('LEFT', YCoordsPanel, 'RIGHT', 3, 0)
		RightCoordDtPanel:Point('CENTER', YCoordsPanel, 'CENTER', 3, 0)
	else
		XCoordsPanel:Point('RIGHT', LocationPlusPanel, 'LEFT', -1, 0)
		XCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', -1, 0)
		YCoordsPanel:Point('LEFT', LocationPlusPanel, 'RIGHT', 1, 0)
		YCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', 1, 0)
		LeftCoordDtPanel:Point('RIGHT', XCoordsPanel, 'LEFT', -1, 0)
		LeftCoordDtPanel:Point('CENTER', XCoordsPanel, 'CENTER', -1, 0)
		RightCoordDtPanel:Point('LEFT', YCoordsPanel, 'RIGHT', 1, 0)
		RightCoordDtPanel:Point('CENTER', YCoordsPanel, 'CENTER', 1, 0)
	end
end

-- Toggle transparency
function AllLocPanelsTransparent()
	local panelsToAddTrans = { LocationPlusPanel, XCoordsPanel, YCoordsPanel, LeftCoordDtPanel, RightCoordDtPanel }
	
	for _, frame in pairs( panelsToAddTrans ) do
		frame:StripTextures(frame) -- yeee baby strip strip!! :D
		if loc_db.loc_trans then
			frame:SetTemplate('Transparent')
			E:UpdateBackdropColors() -- we need this to update the transparency lvl, set on Backdrop Faded Color option
		else
			frame:SetTemplate('Default', true)
		end
	end
end

-- Coord panels
function CreateLocCoordPanels()

	-- X Coord panel
	local coordsX = CreateFrame('Frame', "XCoordsPanel", LocationPlusPanel)
	coordsX:Width(COORDS_WIDTH)
	coordsX:SetHeight(loc_db.dtheight)
	coordsX:SetFrameStrata(LocationPlusPanel:GetFrameStrata())
	coordsX.Text = XCoordsPanel:CreateFontString(nil, "LOW")
	coordsX.Text:SetPoint("CENTER", 1, 0)
	coordsX.Text:SetJustifyH('CENTER')
	coordsX.Text:SetJustifyV('MIDDLE')	

	-- Y Coord panel
	local coordsY = CreateFrame('Frame', "YCoordsPanel", LocationPlusPanel)
	coordsY:Width(COORDS_WIDTH)
	coordsY:SetHeight(loc_db.dtheight)
	coordsY:SetFrameStrata(LocationPlusPanel:GetFrameStrata())
	coordsY.Text = YCoordsPanel:CreateFontString(nil, "LOW")
	coordsY.Text:SetPoint("CENTER", 1, 0)
	coordsY.Text:SetJustifyH('CENTER')
	coordsY.Text:SetJustifyV('MIDDLE')

	local ela = 0

	local cUpdate = function(self,t)
		ela = ela - t
		if ela > 0 then return end
			local x,y = GetPlayerMapPosition("player")
			local xt,yt
			local dig
			if loc_db.loc_dig then
				dig = 2
			else
				dig = 0
			end
			x = tonumber(E:Round(100 * x, dig))
			y = tonumber(E:Round(100 * y, dig))
				if x == 0 and y == 0 then
					if not WorldMapFrame:IsShown() then SetMapToCurrentZone(); end
					coordsX.Text:SetText("-")
					coordsY.Text:SetText("-")
				else
				if x < 10 then
					xt = "0"..x
				else
					xt = x
				end
			if y < 10 then
				yt = "0"..y
			else
				yt = y
			end
			coordsX.Text:SetText(xt)
			coordsY.Text:SetText(yt)
		end
	end
	ela = .2

	XCoordsPanel:SetScript("OnUpdate", cUpdate)

end

function LCoordsDig()
	if loc_db.loc_dig then
		XCoordsPanel:SetWidth(COORDS_WIDTH*1.5)
		YCoordsPanel:SetWidth(COORDS_WIDTH*1.5)
	else
		XCoordsPanel:SetWidth(COORDS_WIDTH)
		YCoordsPanel:SetWidth(COORDS_WIDTH)
	end
end

-- Datatext panels
function CreateLocDTPanels()

	-- Left coords Datatext panel
	left_dtp:SetWidth(loc_db.dtwidth)
	left_dtp:SetHeight(loc_db.dtheight)
	left_dtp:SetFrameStrata('LOW')
	left_dtp:SetParent(LocationPlusPanel) -- set to hide in combat with the Location Panel
	left_dtp:Show()

	-- Right coords Datatext panel
	right_dtp:SetWidth(loc_db.dtwidth)
	right_dtp:SetHeight(loc_db.dtheight)
	right_dtp:SetFrameStrata('LOW')
	right_dtp:SetParent(LocationPlusPanel) -- set to hide in combat with the Location Panel
	right_dtp:Show()
	
end

do
	left_dtp = CreateFrame('Frame', 'LeftCoordDtPanel', E.UIParent)
	DT:RegisterPanel(LeftCoordDtPanel, 1, 'ANCHOR_BOTTOM', 0, -4)
	right_dtp = CreateFrame('Frame', 'RightCoordDtPanel', E.UIParent)
	DT:RegisterPanel(RightCoordDtPanel, 1, 'ANCHOR_BOTTOM', 0, -4)
end

-- How to appear in datatext options
L['RightCoordDtPanel'] = L["LocationPlus Right Panel"];
L['LeftCoordDtPanel'] = L["LocationPlus Left Panel"];

-- Setting default datatexts
P.datatexts.panels.RightCoordDtPanel = 'System'
P.datatexts.panels.LeftCoordDtPanel = 'Durability'

function E:LocPPInit()
	CreateMainLocPanel()
	CreateLocCoordPanels()
		if loc_db.loc_datatxt then
			CreateLocDTPanels()
		end
end

function E:LocDefault()
	if loc_db.lpx == nil then
		loc_db.lpx = (E.screenwidth/2)
	end
	
	if loc_db.lpy == nil then
		loc_db.lpy = (E.screenheight - 22)
	end
	
	if loc_db.dtwidth == nil then
		loc_db.dtwidth = 100
	end	
	
	if loc_db.dtheight == nil then
		loc_db.dtheight = 20
	end	
end

ElvUI_Location:RegisterEvent("PLAYER_ENTERING_WORLD")
ElvUI_Location:SetScript("OnEvent",function(self, event)
    if event == "PLAYER_ENTERING_WORLD" then
		if E.initialized then
			E:LocDefault()
			E:LocPPInit()
			E:LocPlusUpdate()
			E:LocCoordPanelFont()
			self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
			self:RegisterEvent("ZONE_CHANGED")
			self:RegisterEvent("ZONE_CHANGED_INDOORS")
			LocationPlusPanel:RegisterEvent("PLAYER_REGEN_DISABLED")
			LocationPlusPanel:RegisterEvent("PLAYER_REGEN_ENABLED")
			print(L["Location Plus "]..format("v|cff33ffff%s|r",ElvUI_Location.version)..L[" is loaded. Thank you for using it."])
			ElvUI_Location:UnregisterEvent("PLAYER_ENTERING_WORLD")
		end
	end
end)

-- shows when the location panel is CtrlClicked and the DT panels are disabled
StaticPopupDialogs["LOC_DT_DISABLED"] = {
  text = L["LocationPlus Datatexts are disabled. Please go to the options to re-enable them."],
  button1 = "OK",
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  showAlert = true,
  preferredIndex = 3, 
}